@@ -1,58 +1,25 @@
-from __future__ import annotations
-from typing import Optional, Callable, NamedTuple
+from typing import Optional
 import logging
-import pickle
 import sys
 
 import click
 
-from .base import NoChromecastFoundException, \
-  RC_NO_CHROMECAST, LOG_LEVEL, DEFAULT_RETRY_WAIT, \
-  DATA_DIR, NAME, ARGS, NO_DEVICE, RC_NOT_RUNNING
-from .run import get_daemon, run_safe, MprisDaemon
+from .base import RC_NO_CHROMECAST, LOG_LEVEL, \
+  DEFAULT_RETRY_WAIT, RC_NOT_RUNNING, LOG
+from .run import MprisDaemon, DaemonArgs, get_daemon, \
+  run_safe, get_daemon_from_args
 
 
-HELP: str = f"""
+LOG_MODE: str = 'r'
+LOG_END: str = ''
+
+HELP: str = """
 Control casting devices via Linux media controls and desktops.
 
 This daemon connects your casting device directly to the D-Bus media player interface.
 """
 
 
-class DaemonArgs(NamedTuple):
-  name: Optional[str]
-  host: Optional[str]
-  uuid: Optional[str]
-  wait: Optional[float]
-  retry_wait: Optional[float]
-  icon: bool
-  log_level: str
-
-  @property
-  def file(self) -> Path:
-    name, host, uuid, *_ = self
-    device = name or host or uuid or NO_DEVICE
-
-    return ARGS.with_stem(f'{device}-args')
-
-  @staticmethod
-  def load() -> Optional[DaemonArgs]:
-    if ARGS.exists():
-      dump = ARGS.read_bytes()
-      return pickle.loads(dump)
-
-    return None
-
-  @staticmethod
-  def delete():
-    if ARGS.exists():
-      ARGS.unlink()
-
-  def save(self) -> Path:
-    dump = pickle.dumps(self)
-    ARGS.write_bytes(dump)
-
-
 @click.group(help=HELP)
 def cmd():
   pass
@@ -153,11 +120,18 @@ def connect(
     icon,
     log_level
   )
-  daemon = get_daemon(run_safe, *args)
-
-  daemon.start()
   args.save()
 
+  try:
+    daemon = get_daemon_from_args(run_safe, args)
+    daemon.start()
+
+  except Exception as e:
+    logging.exception(e)
+    logging.warning("Error launching daemon.")
+
+    args.delete()
+
 
 @service.command(
   help='Disconnect the background service from the device.'
@@ -165,6 +139,10 @@ def connect(
 def disconnect():
   daemon = get_daemon()
 
+  if not daemon.pid:
+    logging.warning("Daemon isn't running.")
+    sys.exit(RC_NOT_RUNNING)
+
   daemon.stop()
   DaemonArgs.delete()
 
@@ -180,11 +158,24 @@ def reconnect():
     daemon = get_daemon(run_safe, *args)
 
   if not args or not daemon.pid:
-    logging.warn("Daemon isn't running.")
+    logging.warning("Daemon isn't running.")
     sys.exit(RC_NOT_RUNNING)
 
   daemon.restart()
 
 
+@service.command(
+  help='Show the service log.'
+)
+def show():
+  print(f"<Log file: {LOG}>")
+
+  # a large log could crash Python or the system
+  # iterate over the file instead of using Path.read_text()
+  with LOG.open(LOG_MODE) as log:
+    for line in log:
+      print(line, end=LOG_END)
+
+
 if __name__ == "__main__":
   cmd()
