@@ -1,7 +1,9 @@
-from typing import Optional, Callable
+from __future__ import annotations
+from typing import Optional, Callable, NamedTuple
 from time import sleep
 from pathlib import Path
 from functools import partial
+import pickle
 import logging
 import sys
 
@@ -11,45 +13,130 @@ from mpris_server.server import Server
 
 from .base import get_chromecast, Seconds, get_chromecast_via_host, \
   NoChromecastFoundException, LOG_LEVEL, get_chromecast_via_uuid, \
-  DEFAULT_RETRY_WAIT, NoChromecastFoundException, RC_NO_CHROMECAST, \
-  DATA_DIR, NAME, RC_NOT_RUNNING, PID, NO_DEVICE, DEFAULT_WAIT
+  DEFAULT_RETRY_WAIT, RC_NO_CHROMECAST, DATA_DIR, NAME, LOG, \
+  RC_NOT_RUNNING, PID, NO_DEVICE, DEFAULT_WAIT, ARGS
 from .adapter import ChromecastAdapter
 from .listeners import register_mpris_adapter
 
 
+LOG_FILE_MODE: str = 'w'  # create a new log file on each run
+
+
 FuncMaybe = Optional[Callable]
 
 
 class MprisDaemon(RunDaemon):
   target: FuncMaybe = None
+  args: ArgsMaybe = None
 
-  def set_target(self, func: FuncMaybe = None, *args, **kwargs):
+  def set_target(
+    self,
+    func: FuncMaybe = None,
+    *args,
+    **kwargs
+  ):
     if not func:
       self.target = None
       return
 
     self.target = partial(func, *args, **kwargs)
 
+  def set_target_via_args(
+    self,
+    func: FuncMaybe = None,
+    args: ArgsMaybe = None
+  ):
+    if not func:
+      self.target = None
+      return
+
+    self.args = args
+    self.target = partial(func, *args)
+
+  def setup_logging(self):
+    if not self.args:
+      return
+
+    set_log_level(self.args.log_level, file=LOG)
+
   def run(self):
-    if self.target:
-      self.target()
+    if not self.target:
+      return
+
+    self.setup_logging()
+    self.target()
+
+
+class DaemonArgs(NamedTuple):
+  name: Optional[str]
+  host: Optional[str]
+  uuid: Optional[str]
+  wait: Optional[float]
+  retry_wait: Optional[float]
+  icon: bool
+  log_level: str
+
+  @property
+  def file(self) -> Path:
+    name, host, uuid, *_ = self
+    device = name or host or uuid or NO_DEVICE
+
+    return ARGS.with_stem(f'{device}-args')
+
+  @staticmethod
+  def load() -> ArgsMaybe:
+    if ARGS.exists():
+      dump = ARGS.read_bytes()
+      return pickle.loads(dump)
+
+    return None
+
+  @staticmethod
+  def delete():
+    if ARGS.exists():
+      ARGS.unlink()
+
+  def save(self) -> Path:
+    dump = pickle.dumps(self)
+    ARGS.write_bytes(dump)
+
+
+ArgsMaybe = Optional[DaemonArgs]
 
 
 def get_daemon(
   func: FuncMaybe = None,
   *args,
-  pidfile: str = str(PID),
+  _pidfile: str = str(PID),
   **kwargs,
 ) -> MprisDaemon:
-  daemon = MprisDaemon(pidfile=pidfile)
+  daemon = MprisDaemon(pidfile=_pidfile)
   daemon.set_target(func, *args, **kwargs)
 
   return daemon
 
 
-def set_log_level(level: str = LOG_LEVEL):
+def get_daemon_from_args(
+  func: FuncMaybe = None,
+  args: ArgsMaybe = None,
+  _pidfile: str = str(PID),
+) -> MprisDaemon:
+  daemon = MprisDaemon(pidfile=_pidfile)
+  daemon.set_target_via_args(func, args)
+
+  return daemon
+
+
+def set_log_level(
+  level: str = LOG_LEVEL,
+  file: Optional[Path] = None,
+):
   level = level.upper()
-  logging.basicConfig(level=level)
+  logging.basicConfig(
+    level=level,
+    filename=file,
+    filemode=LOG_FILE_MODE
+  )
 
 
 def find_device(
@@ -58,23 +145,23 @@ def find_device(
   uuid: Optional[str],
   retry_wait: Optional[float] = DEFAULT_RETRY_WAIT,
 ) -> Optional[Chromecast]:
-  chromecast: Optional[Chromecast] = None
- 
+  device: Optional[Chromecast] = None
+
   if host:
-    chromecast = get_chromecast_via_host(host, retry_wait)
+    device = get_chromecast_via_host(host, retry_wait)
 
-  if uuid and not chromecast:
-    chromecast = get_chromecast_via_uuid(uuid, retry_wait)
+  if uuid and not device:
+    device = get_chromecast_via_uuid(uuid, retry_wait)
 
-  if name and not chromecast:
-    chromecast = get_chromecast(name, retry_wait)
+  if name and not device:
+    device = get_chromecast(name, retry_wait)
 
   no_identifiers = not (host or name or uuid)
 
   if no_identifiers:
-    chromecast = get_chromecast(retry_wait=retry_wait)
+    device = get_chromecast(retry_wait=retry_wait)
 
-  return chromecast
+  return device
 
 
 def create_adapters_and_server(
@@ -83,16 +170,16 @@ def create_adapters_and_server(
   uuid: Optional[str],
   retry_wait: Optional[float] = DEFAULT_RETRY_WAIT,
 ) -> Optional[Server]:
-  chromecast = find_device(name, host, uuid, retry_wait)
-  
-  if not chromecast:
+  device = find_device(name, host, uuid, retry_wait)
+
+  if not device:
     return None
 
-  chromecast_adapter = ChromecastAdapter(chromecast)
-  mpris = Server(name=chromecast.name, adapter=chromecast_adapter)
+  chromecast_adapter = ChromecastAdapter(device)
+  mpris = Server(name=device.name, adapter=chromecast_adapter)
   mpris.publish()
 
-  register_mpris_adapter(chromecast, mpris, chromecast_adapter)
+  register_mpris_adapter(device, mpris, chromecast_adapter)
 
   return mpris
 
